/*
 * Copyright 2006-2012 The JGUIraffe Team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License")
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.jguiraffe.locators;

import java.net.URL;

/**
 * <p>
 * A specific <code>Locator</code> implementation that can obtain resources from
 * the class path.
 * </p>
 * <p>
 * This class is initialized with a resource name. The <code>getURL()</code>
 * method tries to find this resource from the class path (using the
 * {@link LocatorUtils} class. No caching is performed; each invocation of
 * <code>getURL()</code> will look up the resource. Because a lookup takes the
 * context class loader into account, multiple invocations of
 * <code>getURL()</code> may yield different results.
 * </p>
 * <p>
 * Instances of this class are created using the <code>getInstance()</code>
 * factory method. They are immutable and thus can be shared between multiple
 * threads.
 * </p>
 *
 * @author Oliver Heger
 * @version $Id: ClassPathLocator.java 205 2012-01-29 18:29:57Z oheger $
 */
public final class ClassPathLocator extends AbstractLocator
{
    /** Stores the name of the resource. */
    private final String resourceName;

    /**
     * Creates a new instance of <code>ClassPathLocator</code> and sets the name
     * of the represented resource. To create an instance client code can use
     * the <code>getInstance()</code> factory method.
     *
     * @param resourceName the resource name
     */
    private ClassPathLocator(String resourceName)
    {
        this.resourceName = resourceName;
    }

    /**
     * Returns the name of the resource represented by this locator.
     *
     * @return the resource's name
     */
    public String getResourceName()
    {
        return resourceName;
    }

    /**
     * Returns a <code>ClassPathLocator</code> instance for the specified
     * resource name.
     *
     * @param resourceName the name of the resource (must not be <b>null</b>)
     * @return the instance with this resource name
     * @throws IllegalArgumentException if the resource name is <b>null</b>
     */
    public static ClassPathLocator getInstance(String resourceName)
    {
        if (resourceName == null)
        {
            throw new IllegalArgumentException(
                    "Resource name must not be null!");
        }

        return new ClassPathLocator(resourceName);
    }

    /**
     * Returns the URL for the represented resource. This implementation uses
     * the
     * <code>{@link LocatorUtils#locateResource(String) locateResource()}</code>
     * method of <code>{@link LocatorUtils}</code> to find the resource on the
     * class path. If this fails, an exception is thrown.
     *
     * @return the URL to the resource
     * @throws LocatorException if the resource URL cannot be obtained
     */
    public URL getURL()
    {
        URL result = LocatorUtils.locateResource(getResourceName());
        if (result == null)
        {
            throw new LocatorException("Cannot locate resource "
                    + getResourceName());
        }

        return result;
    }

    /**
     * Compares this object with another one. Two instances of this class are
     * equal if and only if they refer to the same resource name.
     *
     * @param obj the object to be compared to
     * @return a flag whether the objects are equal
     */
    @Override
    public boolean equals(Object obj)
    {
        if (this == obj)
        {
            return true;
        }
        if (!(obj instanceof ClassPathLocator))
        {
            return false;
        }

        ClassPathLocator c = (ClassPathLocator) obj;
        return getResourceName().equals(c.getResourceName());
    }

    /**
     * Returns a hash code for this object.
     *
     * @return a hash code
     */
    @Override
    public int hashCode()
    {
        return getResourceName().hashCode();
    }

    /**
     * Returns a string representation of this object. This string will at least
     * contain the resource name used by this locator.
     *
     * @return a string for this object
     */
    @Override
    public String toString()
    {
        return LocatorUtils.locatorToString(this, "resourceName = "
                + getResourceName());
    }
}
