/*
 * Copyright 2006-2012 The JGUIraffe Team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License")
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.jguiraffe.resources.impl.bundle;

import java.util.Locale;
import java.util.MissingResourceException;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import net.sf.jguiraffe.resources.ResourceGroup;
import net.sf.jguiraffe.resources.ResourceLoader;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * <p>
 * A specialized implementation of the <code>ResourceLoader</code> interface
 * that make use of the default Java resource bundles.
 * </p>
 * <p>
 * This class uses the passed in <code>Locale</code> and resource group name
 * (which is interpreted as string) to find a corresponding resource bundle. For
 * this bundle a wrapping object is returned that implements the
 * <code>{@link de.olix.gen.resources.ResourceGroup ResourceGroup}</code>
 * interface.
 * </p>
 *
 * @author Oliver Heger
 * @version $Id: BundleResourceLoader.java 205 2012-01-29 18:29:57Z oheger $
 */
public class BundleResourceLoader implements ResourceLoader
{
    /** Constant for the initial key buffer size. */
    private static final int BUF_SIZE = 64;

    /** Constant for the key separator character. */
    private static final char KEY_SEPRATOR = '_';

    /** The logger. */
    private final Log log = LogFactory.getLog(getClass());

    /** A cache for the so far created resource groups. */
    private final ConcurrentMap<Object, ResourceGroup> groups;

    /**
     * Creates a new instance of <code>BundleResourceLoader</code>.
     */
    public BundleResourceLoader()
    {
        groups = new ConcurrentHashMap<Object, ResourceGroup>();
    }

    /**
     * Loads the specified resource group from a resource bundle.
     *
     * @param locale the <code>Locale</code>
     * @param name the group's name
     * @return the found resource group
     * @throws MissingResourceException if the group cannot be found
     */
    public ResourceGroup loadGroup(Locale locale, Object name)
    {
        if (name == null)
        {
            throw new IllegalArgumentException(
                    "Resource group name must not be null!");
        }

        String key = keyForGroup(locale, name);
        ResourceGroup group = groups.get(key);
        if (group == null)
        {
            group = createGroup(name.toString(), locale);
            ResourceGroup group2 = groups.putIfAbsent(key, group);
            if (group2 != null)
            {
                group = group2;
            }
        }

        return group;
    }

    /**
     * Constructs a key for the given combination of a locale and a group name.
     * This key is used for storing a group in the cache after it was created.
     *
     * @param locale the locale
     * @param name the group's name
     * @return a key for this group
     */
    protected String keyForGroup(Locale locale, Object name)
    {
        StringBuffer buf = new StringBuffer(BUF_SIZE);
        buf.append(name).append(KEY_SEPRATOR);
        if (locale != null)
        {
            buf.append(locale);
        }
        return buf.toString();
    }

    /**
     * Creates a <code>ResourceGroup</code> object from the specified bundle.
     *
     * @param locale the <code>Locale</code>
     * @param name the group's name
     * @return the new resource group
     * @throws MissingResourceException if the bundle cannot be found
     */
    protected ResourceGroup createGroup(String name, Locale locale)
            throws MissingResourceException
    {
        if (log.isInfoEnabled())
        {
            log
                    .info("Loading resource group " + name + " for locale "
                            + locale);
        }
        return new BundleResourceGroup(name, locale);
    }
}
